//Double_t kRad2Deg = 180./TMath::Pi();

#include <float.h>

Double_t GetMin(TH1 *h)
{
    Double_t min = FLT_MAX;
    for (int i=1; i<=h->GetXaxis()->GetNbins(); i++)
    {
        if (h->GetBinContent(i)<min && h->GetBinContent(i)!=0)
            min = h->GetBinContent(i);
    }
    return min;
}

void GetRange(TChain *chain, const char *name, Int_t &nbins, Double_t &min, Double_t &max, Double_t conv=1)
{
    TString str("MPhoton.MParticle.");

    MPhoton *p = new MPhoton;

    chain->SetBranchAddress("MPhoton.", &p);

    min =  FLT_MAX;
    max = -FLT_MAX;

    Int_t n = chain->GetEntries();
    for (int i=0; i<n; i++)
    {
        chain->GetEntry(i);

        TLeaf *leaf = chain->GetLeaf(str+name);
        if (!leaf)
            return 0;

        Double_t val = leaf->GetValue();

        if (p->IsPrimary())
            continue;

        if (val < min)
            min = val;
        if (val > max)
            max = val;
    }

    min *= conv;
    max *= conv;

    chain.GetPlayer();
    TTreePlayer &play = *chain.GetPlayer();

    Int_t n;
    play.FindGoodLimits(nbins, n, min, max, kFALSE);
    nbins = n;
}

void draw1h1426(Double_t E, Double_t plot=1)
{
    plot=2;
    Double_t level = log10(E);
    level -= log10(5.73e-0*pow( 0.39e3, plot));

    cout << "Level: " << level << endl;

    TPolyMarker *m = new TPolyMarker(7);
    /*
     m->SetPoint(0,  0.28e3, 1.86e-0*pow( 0.28e3, plot)*level);
     m->SetPoint(1,  0.39e3, 5.73e-0*pow( 0.39e3, plot)*level);
     m->SetPoint(2,  0.80e3, 1.22e-0*pow( 0.80e3, plot)*level);
     m->SetPoint(3,  1.55e3, 5.10e-2*pow( 1.55e3, plot)*level);
     m->SetPoint(4,  2.82e3, 1.50e-2*pow( 2.82e3, plot)*level);
     m->SetPoint(5,  5.33e3, 7.70e-3*pow( 5.33e3, plot)*level);
     m->SetPoint(6, 10.00e3, 1.20e-4*pow(10.00e3, plot)*level);
     */
    m->SetPoint(0, log10( 0.28e3), log10(1.86e-0*pow( 0.28e3, plot))+level);
    m->SetPoint(1, log10( 0.39e3), log10(5.73e-0*pow( 0.39e3, plot))+level);
    m->SetPoint(2, log10( 0.80e3), log10(1.22e-0*pow( 0.80e3, plot))+level);
    m->SetPoint(3, log10( 1.55e3), log10(5.10e-2*pow( 1.55e3, plot))+level);
    m->SetPoint(4, log10( 2.82e3), log10(1.50e-2*pow( 2.82e3, plot))+level);
    m->SetPoint(5, log10( 5.33e3), log10(7.70e-3*pow( 5.33e3, plot))+level);
    m->SetPoint(6, log10(10.00e3), log10(1.20e-4*pow(10.00e3, plot))+level);
    m->SetMarkerStyle(kOpenStar);
//    m->SetMarkerSize(1);
    m->Draw();
}

void analp()
{
    TChain chain("Photons");
    /*
     chain.Add("delme3.root");
     chain.Add("delme3B.root");
     chain.Add("delme3C.root");
     chain.Add("delme3D.root");
     chain.Add("delme3E.root");
     */
    chain.Add("delme3H.root");

    Int_t nbins =  24;
    Double_t lo = 1e2;
    Double_t hi = 1e6;

    MBinning binsx;
    binsx.SetEdgesLog(nbins, lo, hi);

    // ------------------------------

    Double_t cutoff = 1e12;

    Double_t alpha = -1;
    Double_t plot  =  1;

    Int_t nbins = 16;

    // ------------------------------

    Int_t n = chain.GetEntries();

    cout << endl << "Found " << n << " entries." << endl;

    if (n==0)
        return;

    MBinning binspolx;
    MBinning binspola;
    MBinning binspolr;
    binspolx.SetEdges(16, -180, 180);

    Double_t min, max;
    GetRange(&chain, "fTheta", nbins, min, max, kRad2Deg*60);
    binspola.SetEdges(nbins, min, max*1.1);
    cout << "fTheta, " << nbins << ": " << min << " ... " << max << " [']" << endl;

    GetRange(&chain, "fR", nbins, min, max);
    binspolr.SetEdges(nbins, min, max*1.1);
    cout << "fR,     " << nbins << ": " << min << " ... " << max << " [kpc]" << endl;

    TH1D h, h2, h3;
    TH1D prim;
    MH::SetBinning(&h, &binsx);
    MH::SetBinning(&h2, &binsx);
    MH::SetBinning(&h3, &binsx);
    MH::SetBinning(&prim, &binsx);

    TH2D r, r2, r3;
    TH2D a, a2, a3;
    MH::SetBinning(&a, &binspolx, &binspola);
    MH::SetBinning(&r, &binspolx, &binspolr);
    MH::SetBinning(&a2, &binspolx, &binspola);
    MH::SetBinning(&r2, &binspolx, &binspolr);
    MH::SetBinning(&a3, &binspolx, &binspola);
    MH::SetBinning(&r3, &binspolx, &binspolr);

    MPhoton *p = new MPhoton;
    chain.SetBranchAddress("MPhoton.", &p);

    chain.GetEntry(0);
    if (!p->IsPrimary())
        return;

    Double_t z = p->GetZ();
    Double_t R = MParticle::RofZ(&z);

    cout << "Z = " << z << endl;
    cout << "R = " << R << "kpc" << endl;

    Double_t weight = 0;

    Bool_t isprim = kFALSE;
    Bool_t ignore = kFALSE;
    for (int i=0; i<n; i++)
    {
        chain.GetEntry(i);

        Double_t Ep = p->GetEnergy();

        if (p->IsPrimary())
        {
            if (Ep>cutoff)
            {
                ignore = kTRUE;
                continue;
            }
            ignore = kFALSE;
            weight = pow(Ep, alpha);
            prim.Fill(Ep, pow(Ep, plot+alpha));
            isprim = kTRUE;
            continue;
        }

        if (ignore)
            continue;

        h.Fill(Ep, pow(Ep,plot) * weight);

        Double_t phi = fmod(p->GetPhi()*kRad2Deg+180, 360)-180;
        Double_t psi = fmod(p->GetPsi()*kRad2Deg+180, 360)-180;

        r.Fill(phi, isprim?0:p->GetR(),                 weight);
        a.Fill(psi, isprim?0:p->GetTheta()*kRad2Deg*60, weight);

        if (isprim)
        {
            h3.Fill(Ep, pow(Ep,plot) * weight);
            r3.Fill(phi, 0,                 weight);
            a3.Fill(psi, 0, weight);
            isprim = kFALSE;
            continue;
        }

        h2.Fill(Ep, pow(Ep,plot) * weight);
        r2.Fill(phi, p->GetR(),                 weight);
        a2.Fill(psi, p->GetTheta()*kRad2Deg*60, weight);
    }

    delete p;

    TH1 *g=r.ProjectionX();
    cout << "Mean fPhi: " << g->GetMean() << " +- " << g->GetRMS() << endl;
    delete g;
    g=a.ProjectionX();
    cout << "Mean fPsi: " << g->GetMean() << " +- " << g->GetRMS() <<  endl;
    delete g;

    gStyle->SetOptStat(10);

    TLine line;

//    delete gROOT->FindObject("Analysis Arrival");
    c = MH::MakeDefCanvas("Analysis Arrival", "");
    c->Divide(2,2);

    gStyle->SetPalette(1, 0);

    c->cd(1);
    r.SetTitle(" Distance from Observer ");
    r.GetXaxis()->SetLabelOffset(-0.015);
    r.GetXaxis()->SetTitleOffset(1.1);
    r.GetXaxis()->SetRangeUser(1e4, 1e9);
    r.SetXTitle("\\Phi [\\circ]");
    r.SetYTitle("R [kpc]");
    TPad &pad = *gPad;
    pad.Divide(2,2);
    pad.cd(1);
    gPad->SetLogy();
    gPad->SetLogz();
    gPad->SetTheta(0);
    gPad->SetPhi(90);
    g = r.DrawCopy("surf1pol");
    pad.cd(2);
    gPad->SetLogy();
    gPad->SetLogz();
    gPad->SetTheta(70);
    gPad->SetPhi(90);
    g->Draw("surf1pol");
    pad.cd(3);
    gPad->SetLogy();
    gPad->SetLogz();
    gPad->SetTheta(90);
    gPad->SetPhi(90);
    g->Draw("surf1pol");
    pad.cd(4);
    gPad->SetLogy();
    gPad->SetLogz();
    gPad->SetTheta(20);
    gPad->SetPhi(90);
    g->Draw("surf1pol");

    c->cd(2);
    a.SetTitle(" Hit Angle for Observer ");
    a.GetXaxis()->SetLabelOffset(-0.015);
    a.GetXaxis()->SetTitleOffset(1.1);
    a.GetXaxis()->SetRangeUser(1e4, 1e9);
    a.SetXTitle("\\Psi [\\circ]");
    a.SetYTitle("\\Theta [']");
    TPad &pad = *gPad;
    pad.Divide(2,2);
    pad.cd(1);
    gPad->SetLogy();
    gPad->SetLogz();
    gPad->SetTheta(0);
    gPad->SetPhi(90);
    g = a.DrawCopy("surf1pol");
    pad.cd(2);
    gPad->SetLogy();
    gPad->SetLogz();
    gPad->SetTheta(70);
    gPad->SetPhi(90);
    g->Draw("surf1pol");
    pad.cd(3);
    gPad->SetLogy();
    gPad->SetLogz();
    gPad->SetTheta(90);
    gPad->SetPhi(90);
    g->Draw("surf1pol");
    pad.cd(4);
    gPad->SetLogy();
    gPad->SetLogz();
    gPad->SetTheta(20);
    gPad->SetPhi(90);
    g->Draw("surf1pol");

    c->cd(3);
    gPad->SetLogy();
    g=r.ProjectionY();
    g->SetDirectory(NULL);
    TH1 *g2=r2.ProjectionY();
    g->SetMinimum(GetMin(g2)/2);
    g->SetBit(kCanDelete);
    g->SetTitle(" Hit Observers Plain ");
    g->GetXaxis()->SetLabelOffset(0);
    g->GetXaxis()->SetTitleOffset(1.1);
    g->GetYaxis()->SetTitleOffset(1.3);
    g->SetXTitle("R [kpc]");
    g->SetYTitle("Counts");
    g->Draw();
    g2->SetDirectory(NULL);
    g2->SetBit(kCanDelete);
    g2->SetLineColor(kBlue);
    g2->Draw("same");
    g=r3.ProjectionY();
    g->SetDirectory(NULL);
    g->SetBit(kCanDelete);
    g->SetLineColor(kGreen);
    g->Draw("same");

    c->cd(4);
    gPad->SetLogy();
    g=a.ProjectionY();
    g->SetMinimum(GetMin(g)/2);
    g->SetDirectory(NULL);
    g->SetBit(kCanDelete);
    g->SetTitle("Hit Angle");
    g->GetXaxis()->SetLabelOffset(0);
    g->GetXaxis()->SetTitleOffset(1.1);
    g->GetYaxis()->SetTitleOffset(1.3);
    g->SetXTitle("\\Phi [']");
    g->SetYTitle("Counts");
    g->Draw();
    g=a2.ProjectionY();
    g->SetDirectory(NULL);
    g->SetBit(kCanDelete);
    g->SetLineColor(kBlue);
    g->Draw("same");
    g=a3.ProjectionY();
    g->SetDirectory(NULL);
    g->SetBit(kCanDelete);
    g->SetLineColor(kGreen);
    g->Draw("same");

  //  delete gROOT->FindObject("Analysis Photons");
    TCanvas *c = MH::MakeDefCanvas("Analysis Photons", "", 580, 870);
    c->Divide(1,2);

    c->cd(1);
    gPad->SetLogx();
    gPad->SetLogy();
    h.SetFillStyle(0);
    h.SetName("Photons");
    h.SetTitle(Form(" E^{%.1f}  Spectra ", alpha));
    h.SetXTitle("E\\gamma [GeV]");
    h.SetYTitle(Form("E^{%.1f} * Counts", plot));
    h.GetXaxis()->SetLabelOffset(-0.015);
    h.GetXaxis()->SetTitleOffset(1.1);
    h.SetMarkerStyle(kMultiply);
    prim.SetFillStyle(0);
    prim.SetMarkerStyle(kPlus);
    prim.SetMarkerColor(kRed);
    prim.SetLineColor(kRed);
    TH1 &g1=*h.DrawCopy("P");
    g2=*prim.DrawCopy("Psame");
    g2->Draw("Csame");
    g1.Draw("Csame");
    h2.SetFillStyle(0);
    h2.SetName("SecPhotons");
    h2.SetMarkerStyle(kMultiply);
    h2.SetMarkerColor(kBlue);
    h2.SetLineColor(kBlue);
    TH1 &g3=*h2.DrawCopy("Psame");
    g3.Draw("Csame");
    h3.SetFillStyle(0);
    h3.SetName("PrimPhotons");
    h3.SetMarkerStyle(kMultiply);
    h3.SetMarkerColor(kGreen);
    h3.SetLineColor(kGreen);
    TH1 &g4=*h3.DrawCopy("Psame");
    g4.Draw("Csame");

    draw1h1426(prim.GetBinContent(2),plot);

    c->cd(2);
    gPad->SetLogx();
    TH1D div;
    MH::SetBinning(&div, &binsx);
    div.Divide(&h, &prim);
    div.SetTitle("Spectrum / Primary Spectrum");
    div.GetXaxis()->SetLabelOffset(-0.015);
    div.GetXaxis()->SetTitleOffset(1.1);
    div.SetXTitle("E\\gamma [GeV]");
    div.SetYTitle("Ratio");
    line.SetLineColor(kBlue);
    line.SetLineWidth(1);
    div.DrawCopy();
    line.DrawLine(log10(lo), 1, log10(hi), 1);
}
